/**
 * DevExtreme (renovation/preact_wrapper/check_box.js)
 * Version: 20.2.1 (build 20239-0312)
 * Build date: Wed Aug 26 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.default = void 0;
var _component = _interopRequireDefault(require("./component"));
var _validation_engine = _interopRequireDefault(require("../../ui/validation_engine"));
var _extend = require("../../core/utils/extend");
var _overlay = _interopRequireDefault(require("../../ui/overlay"));
var _window = require("../../core/utils/window");
var _string = require("../../core/utils/string");
var _renderer = _interopRequireDefault(require("../../core/renderer"));
var _guid = _interopRequireDefault(require("../../core/guid"));
var _element_data = require("../../core/element_data");
var _callbacks = _interopRequireDefault(require("../../core/utils/callbacks"));
var _position = require("../../core/utils/position");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _typeof(obj) {
    "@babel/helpers - typeof";
    if ("function" === typeof Symbol && "symbol" === typeof Symbol.iterator) {
        _typeof = function(obj) {
            return typeof obj
        }
    } else {
        _typeof = function(obj) {
            return obj && "function" === typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj
        }
    }
    return _typeof(obj)
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}

function _get(target, property, receiver) {
    if ("undefined" !== typeof Reflect && Reflect.get) {
        _get = Reflect.get
    } else {
        _get = function(target, property, receiver) {
            var base = _superPropBase(target, property);
            if (!base) {
                return
            }
            var desc = Object.getOwnPropertyDescriptor(base, property);
            if (desc.get) {
                return desc.get.call(receiver)
            }
            return desc.value
        }
    }
    return _get(target, property, receiver || target)
}

function _superPropBase(object, property) {
    while (!Object.prototype.hasOwnProperty.call(object, property)) {
        object = _getPrototypeOf(object);
        if (null === object) {
            break
        }
    }
    return object
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function")
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        _setPrototypeOf(subClass, superClass)
    }
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}

function _createSuper(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct();
    return function() {
        var result, Super = _getPrototypeOf(Derived);
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget)
        } else {
            result = Super.apply(this, arguments)
        }
        return _possibleConstructorReturn(this, result)
    }
}

function _possibleConstructorReturn(self, call) {
    if (call && ("object" === _typeof(call) || "function" === typeof call)) {
        return call
    }
    return _assertThisInitialized(self)
}

function _assertThisInitialized(self) {
    if (void 0 === self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return self
}

function _isNativeReflectConstruct() {
    if ("undefined" === typeof Reflect || !Reflect.construct) {
        return false
    }
    if (Reflect.construct.sham) {
        return false
    }
    if ("function" === typeof Proxy) {
        return true
    }
    try {
        Date.prototype.toString.call(Reflect.construct(Date, [], function() {}));
        return true
    } catch (e) {
        return false
    }
}

function _getPrototypeOf(o) {
    _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function(o) {
        return o.__proto__ || Object.getPrototypeOf(o)
    };
    return _getPrototypeOf(o)
}
var INVALID_MESSAGE_AUTO = "dx-invalid-message-auto";
var VALIDATION_TARGET = "dx-validation-target";
var VALIDATION_MESSAGE_MIN_WIDTH = 100;

function getValidationErrorMessage(validationErrors) {
    var validationErrorMessage = "";
    if (validationErrors) {
        validationErrors.forEach(function(err) {
            if (err.message) {
                validationErrorMessage += (validationErrorMessage ? "<br />" : "") + (0, _string.encodeHtml)(err.message)
            }
        })
    }
    return validationErrorMessage
}
var CheckBox = function(_Component) {
    _inherits(CheckBox, _Component);
    var _super = _createSuper(CheckBox);

    function CheckBox() {
        _classCallCheck(this, CheckBox);
        return _super.apply(this, arguments)
    }
    _createClass(CheckBox, [{
        key: "getProps",
        value: function() {
            var _this = this;
            var props = _get(_getPrototypeOf(CheckBox.prototype), "getProps", this).call(this);
            props.onFocusIn = function() {
                var isValidationMessageShownOnFocus = "auto" === _this.option("validationMessageMode");
                if (isValidationMessageShownOnFocus) {
                    _this._$validationMessage && _this._$validationMessage.removeClass(INVALID_MESSAGE_AUTO);
                    clearTimeout(_this.showValidationMessageTimeout);
                    _this.showValidationMessageTimeout = setTimeout(function() {
                        _this._$validationMessage && _this._$validationMessage.addClass(INVALID_MESSAGE_AUTO)
                    }, 150)
                }
            };
            props.saveValueChangedEvent = function(e) {
                _this._valueChangeEventInstance = e
            };
            return props
        }
    }, {
        key: "_init",
        value: function() {
            _get(_getPrototypeOf(CheckBox.prototype), "_init", this).call(this);
            (0, _element_data.data)(this.$element()[0], VALIDATION_TARGET, this);
            this.validationRequest = (0, _callbacks.default)();
            this.showValidationMessageTimeout = null;
            this._valueChangeAction = this._createActionByOption("onValueChanged", {
                excludeValidators: ["disabled", "readOnly"]
            })
        }
    }, {
        key: "_getDefaultOptions",
        value: function() {
            return (0, _extend.extend)(_get(_getPrototypeOf(CheckBox.prototype), "_getDefaultOptions", this).call(this), {
                validationMessageOffset: {
                    h: 0,
                    v: 0
                },
                validationTooltipOptions: {}
            })
        }
    }, {
        key: "_setValidationMessageMaxWidth",
        value: function() {
            if (!this._validationMessage) {
                return
            }
            if (0 === this._getValidationMessageTarget().outerWidth()) {
                this._validationMessage.option("maxWidth", "100%");
                return
            }
            var validationMessageMaxWidth = Math.max(VALIDATION_MESSAGE_MIN_WIDTH, this._getValidationMessageTarget().outerWidth());
            this._validationMessage.option("maxWidth", validationMessageMaxWidth)
        }
    }, {
        key: "_renderValidationState",
        value: function() {
            var isValid = this.option("isValid") && "invalid" !== this.option("validationStatus");
            var validationMessageMode = this.option("validationMessageMode");
            var $element = this.$element();
            var validationErrors = this.option("validationErrors");
            if (!validationErrors && this.option("validationError")) {
                validationErrors = [this.option("validationError")]
            }
            if (!(0, _window.hasWindow)()) {
                return
            }
            if (this._$validationMessage) {
                this._$validationMessage.remove();
                $element.attr("aria-describedby", null);
                this._$validationMessage = null
            }
            var validationErrorMessage = getValidationErrorMessage(validationErrors);
            if (!isValid && validationErrorMessage) {
                this._$validationMessage = (0, _renderer.default)("<div>").addClass("dx-invalid-message").html(validationErrorMessage).appendTo($element);
                var validationTarget = this._getValidationMessageTarget();
                this._validationMessage = this._createComponent(this._$validationMessage, _overlay.default, (0, _extend.extend)({
                    integrationOptions: {},
                    templatesRenderAsynchronously: false,
                    target: validationTarget,
                    shading: false,
                    width: "auto",
                    height: "auto",
                    container: $element,
                    position: this._getValidationMessagePosition("below"),
                    closeOnOutsideClick: false,
                    closeOnTargetScroll: false,
                    animation: null,
                    visible: true,
                    propagateOutsideClick: true,
                    _checkParentVisibility: false
                }, this._options.cache("validationTooltipOptions")));
                this._$validationMessage.toggleClass(INVALID_MESSAGE_AUTO, "auto" === validationMessageMode).toggleClass("dx-invalid-message-always", "always" === validationMessageMode);
                var messageId = "dx-".concat(new _guid.default);
                this._validationMessage.$content().addClass("dx-invalid-message-content").attr("id", messageId);
                $element.attr("aria-describedby", messageId);
                this._setValidationMessageMaxWidth();
                this._bindInnerWidgetOptions(this._validationMessage, "validationTooltipOptions")
            }
        }
    }, {
        key: "_bindInnerWidgetOptions",
        value: function(innerWidget, optionsContainer) {
            var _this2 = this;
            var syncOptions = function() {
                return _this2._options.silent(optionsContainer, (0, _extend.extend)({}, innerWidget.option()))
            };
            syncOptions();
            innerWidget.on("optionChanged", syncOptions)
        }
    }, {
        key: "_getValidationMessageTarget",
        value: function() {
            return this.$element()
        }
    }, {
        key: "_getValidationMessagePosition",
        value: function(positionRequest) {
            var rtlEnabled = this.option("rtlEnabled");
            var messagePositionSide = (0, _position.getDefaultAlignment)(rtlEnabled);
            var messageOriginalOffset = this.option("validationMessageOffset");
            var messageOffset = {
                h: messageOriginalOffset.h,
                v: messageOriginalOffset.v
            };
            var verticalPositions = "below" === positionRequest ? [" top", " bottom"] : [" bottom", " top"];
            if (rtlEnabled) {
                messageOffset.h = -messageOffset.h
            }
            if ("below" !== positionRequest) {
                messageOffset.v = -messageOffset.v
            }
            return {
                offset: messageOffset,
                boundary: this.option("validationBoundary"),
                my: messagePositionSide + verticalPositions[0],
                at: messagePositionSide + verticalPositions[1],
                collision: "none flip"
            }
        }
    }, {
        key: "_optionChanged",
        value: function(option) {
            var _this$_valueChangeAct;
            var _ref = option || {},
                name = _ref.name,
                previousValue = _ref.previousValue,
                value = _ref.value;
            if (name && this._getActionConfigs()[name]) {
                this._addAction(name)
            }
            switch (name) {
                case "value":
                    if (value !== previousValue) {
                        this.validationRequest.fire({
                            value: value,
                            editor: this
                        })
                    }
                    null === (_this$_valueChangeAct = this._valueChangeAction) || void 0 === _this$_valueChangeAct ? void 0 : _this$_valueChangeAct.call(this, {
                        element: this.$element(),
                        previousValue: previousValue,
                        value: value,
                        event: this._valueChangeEventInstance
                    });
                    this._valueChangeEventInstance = null;
                    break;
                case "onValueChanged":
                    this._valueChangeAction = this._createActionByOption("onValueChanged", {
                        excludeValidators: ["disabled", "readOnly"]
                    });
                    break;
                case "isValid":
                case "validationError":
                    this.option(_validation_engine.default.synchronizeValidationOptions(option, this.option()));
                    break;
                case "validationErrors":
                case "validationStatus":
                    this.option(_validation_engine.default.synchronizeValidationOptions(option, this.option()));
                    this._renderValidationState();
                    break;
                default:
                    this.$element().toggleClass("dx-invalid", !this.option("isValid"));
                    _get(_getPrototypeOf(CheckBox.prototype), "_optionChanged", this).call(this, option)
            }
            this._invalidate()
        }
    }, {
        key: "reset",
        value: function() {
            var defaultOptions = this._getDefaultOptions();
            this.option("value", defaultOptions.value)
        }
    }, {
        key: "_dispose",
        value: function() {
            _get(_getPrototypeOf(CheckBox.prototype), "_dispose", this).call(this);
            (0, _element_data.data)(this.element(), VALIDATION_TARGET, null);
            clearTimeout(this.showValidationMessageTimeout)
        }
    }]);
    return CheckBox
}(_component.default);
exports.default = CheckBox;
module.exports = exports.default;
