/**
 * DevExtreme (ui/scheduler/appointmentSettingsGenerator.js)
 * Version: 20.2.1 (build 20239-0312)
 * Build date: Wed Aug 26 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.default = void 0;
var _date = _interopRequireDefault(require("../../core/utils/date"));
var _extend = require("../../core/utils/extend");
var _recurrence = require("./recurrence");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}
var AppointmentSettingsGenerator = function() {
    function AppointmentSettingsGenerator(scheduler) {
        _classCallCheck(this, AppointmentSettingsGenerator);
        this.scheduler = scheduler
    }
    _createClass(AppointmentSettingsGenerator, [{
        key: "create",
        value: function(appointmentData) {
            var _this = this;
            var adapter = this.scheduler.createAppointmentAdapter(appointmentData);
            var recurrenceRule = adapter.recurrenceRule;
            var dateRange = this.scheduler._workSpace.getDateRange();
            var allDay = this.scheduler.appointmentTakesAllDay(appointmentData);
            var startViewDate = this.scheduler.appointmentTakesAllDay(appointmentData) ? _date.default.trimTime(new Date(dateRange[0])) : dateRange[0];
            var renderingStrategy = this.scheduler.getLayoutManager().getRenderingStrategyInstance();
            var firstDayOfWeek = this.scheduler.getFirstDayOfWeek();
            var minRecurrenceDate = this.scheduler.option("timeZone") ? this.scheduler.timeZoneCalculator.createDate(startViewDate, {
                path: "fromGrid"
            }) : startViewDate;
            var maxRecurrenceDate = this.scheduler.option("timeZone") ? this.scheduler.timeZoneCalculator.createDate(dateRange[1], {
                path: "fromGrid"
            }) : dateRange[1];
            var recurrenceOptions = {
                rule: recurrenceRule,
                exception: adapter.recurrenceException,
                start: adapter.startDate,
                end: adapter.endDate,
                min: minRecurrenceDate,
                max: maxRecurrenceDate,
                firstDayOfWeek: firstDayOfWeek
            };
            var createAppointmentInfo = function(startDate, endDate, source) {
                return {
                    startDate: startDate,
                    endDate: endDate,
                    source: source
                }
            };
            var appointmentDuration = adapter.endDate ? adapter.endDate.getTime() - adapter.startDate.getTime() : 0;
            var appointmentList = this._createRecurrenceAppointments(recurrenceOptions, appointmentDuration);
            if (0 === appointmentList.length) {
                appointmentList.push({
                    startDate: adapter.startDate,
                    endDate: adapter.endDate
                })
            }
            var gridAppointmentList = appointmentList.map(function(source) {
                var startDate = _this.scheduler.timeZoneCalculator.createDate(source.startDate, {
                    appointmentTimeZone: adapter.startDateTimeZone,
                    path: "toGrid"
                });
                var endDate = _this.scheduler.timeZoneCalculator.createDate(source.endDate, {
                    appointmentTimeZone: adapter.endDateTimeZone,
                    path: "toGrid"
                });
                return createAppointmentInfo(startDate, endDate, source)
            });
            gridAppointmentList = this._cropAppointmentsByStartDayHour(gridAppointmentList, appointmentData);
            if (renderingStrategy.needSeparateAppointment(allDay)) {
                var longParts = [];
                var resultDates = [];
                gridAppointmentList.forEach(function(gridAppointment) {
                    var maxDate = new Date(dateRange[1]);
                    var endDateOfPart = renderingStrategy.normalizeEndDateByViewEnd(appointmentData, gridAppointment.endDate);
                    longParts = _date.default.getDatesOfInterval(gridAppointment.startDate, endDateOfPart, {
                        milliseconds: _this.scheduler.getWorkSpace().getIntervalDuration(allDay)
                    });
                    var newArr = longParts.filter(function(el) {
                        return new Date(el) < maxDate
                    }).map(function(date) {
                        return createAppointmentInfo(date, new Date(new Date(date).setMilliseconds(appointmentDuration)), gridAppointment.source)
                    });
                    resultDates = resultDates.concat(newArr)
                });
                gridAppointmentList = resultDates
            }
            var itemResources = this.scheduler._resourcesManager.getResourcesFromItem(appointmentData);
            allDay = this.scheduler.appointmentTakesAllDay(appointmentData) && this.scheduler._workSpace.supportAllDayRow();
            return this._createAppointmentInfos(gridAppointmentList, itemResources, allDay)
        }
    }, {
        key: "_createRecurrenceAppointments",
        value: function(option, duration) {
            var result = (0, _recurrence.getRecurrenceProcessor)().generateDates(option);
            return result.map(function(date) {
                return {
                    startDate: new Date(date),
                    endDate: new Date(new Date(date).setMilliseconds(duration))
                }
            })
        }
    }, {
        key: "_cropAppointmentsByStartDayHour",
        value: function(appointments, appointmentData) {
            var _this2 = this;
            var startDayHour = this.scheduler._getCurrentViewOption("startDayHour");
            var firstViewDate = this.scheduler._workSpace.getStartViewDate();
            return appointments.map(function(appointment) {
                var startDate = new Date(appointment.startDate);
                var resultDate = new Date(appointment.startDate);
                if (_this2.scheduler.appointmentTakesAllDay(appointmentData)) {
                    resultDate = _date.default.normalizeDate(startDate, firstViewDate)
                } else {
                    if (startDate < firstViewDate) {
                        startDate = firstViewDate
                    }
                    resultDate = _date.default.normalizeDate(appointment.startDate, startDate)
                }
                appointment.startDate = _date.default.roundDateByStartDayHour(resultDate, startDayHour);
                return appointment
            })
        }
    }, {
        key: "_createAppointmentInfos",
        value: function(gridAppointments, appointmentResources, allDay) {
            var _this3 = this;
            var result = [];
            var _loop = function(i) {
                var coordinates = _this3.scheduler._workSpace.getCoordinatesByDateInGroup(gridAppointments[i].startDate, appointmentResources, allDay);
                coordinates.forEach(function(coordinate) {
                    (0, _extend.extend)(coordinate, {
                        info: {
                            appointment: gridAppointments[i],
                            sourceAppointment: gridAppointments[i].source
                        }
                    })
                });
                result = result.concat(coordinates)
            };
            for (var i = 0; i < gridAppointments.length; i++) {
                _loop(i)
            }
            return result
        }
    }]);
    return AppointmentSettingsGenerator
}();
exports.default = AppointmentSettingsGenerator;
module.exports = exports.default;
