/**
 * DevExtreme (ui/scheduler/workspaces/ui.scheduler.virtual_scrolling.js)
 * Version: 20.2.1 (build 20239-0312)
 * Build date: Wed Aug 26 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.default = void 0;

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}
var ROW_HEIGHT = 50;
var VirtualScrolling = function() {
    function VirtualScrolling(workspace, viewportHeight, scrollable) {
        _classCallCheck(this, VirtualScrolling);
        this._workspace = workspace;
        this._viewportHeight = viewportHeight;
        this._scrollable = scrollable;
        this._init()
    }
    _createClass(VirtualScrolling, [{
        key: "getWorkspace",
        value: function() {
            return this._workspace
        }
    }, {
        key: "getViewportHeight",
        value: function() {
            return this._viewportHeight
        }
    }, {
        key: "getScrollable",
        value: function() {
            return this._scrollable
        }
    }, {
        key: "getRowHeight",
        value: function() {
            return ROW_HEIGHT
        }
    }, {
        key: "getState",
        value: function() {
            return this._state
        }
    }, {
        key: "_getPageSize",
        value: function() {
            return Math.ceil(this.getViewportHeight() / this.getRowHeight())
        }
    }, {
        key: "_getOutlineCount",
        value: function() {
            return Math.floor(this._getPageSize() / 2)
        }
    }, {
        key: "_init",
        value: function() {
            var scrollOffset = {
                top: 0,
                left: 0
            };
            this._state = {
                pageSize: this._getPageSize(),
                scrollOffset: scrollOffset,
                startIndex: -1,
                rowCount: 0,
                topVirtualRowCount: 0,
                bottomVirtualRowCount: 0,
                topOutlineCount: 0,
                bottomOutlineCount: 0,
                topVirtualRowHeight: 0,
                bottomVirtualRowHeight: 0,
                topOutlineHeight: 0,
                bottomOutlineHeight: 0
            };
            this._updateState(scrollOffset);
            this._attachScrollableEvent()
        }
    }, {
        key: "_attachScrollableEvent",
        value: function() {
            var _this = this;
            var scrollable = this.getScrollable();
            var onScroll = scrollable.option("onScroll");
            scrollable.option("onScroll", function(e) {
                null === onScroll || void 0 === onScroll ? void 0 : onScroll.apply(scrollable, [e]);
                var scrollOffset = null === e || void 0 === e ? void 0 : e.scrollOffset;
                if (scrollOffset && _this._updateState(scrollOffset)) {
                    _this.getWorkspace().renderRWorkspace()
                }
            })
        }
    }, {
        key: "_updateState",
        value: function(scrollOffset) {
            var state = this.getState();
            var top = scrollOffset.top;
            var currentStartIndex = state.startIndex;
            var rowHeight = this.getRowHeight();
            state.scrollOffset = scrollOffset;
            state.startIndex = Math.floor(top / rowHeight);
            if (currentStartIndex !== state.startIndex) {
                var topRowsInfo = this._calcTopRowsInfo(scrollOffset);
                var topRowsDelta = this._calcTopRowsDelta(topRowsInfo);
                var _this$_calcBottomRows = this._calcBottomRowsInfo(topRowsDelta),
                    bottomOutlineCount = _this$_calcBottomRows.bottomOutlineCount,
                    bottomVirtualRowCount = _this$_calcBottomRows.bottomVirtualRowCount,
                    rowCountWithBottom = _this$_calcBottomRows.rowCountWithBottom;
                var topVirtualRowCount = topRowsInfo.topVirtualRowCount,
                    topOutlineCount = topRowsInfo.topOutlineCount;
                var rowCount = topOutlineCount + rowCountWithBottom + bottomOutlineCount;
                state.startIndex -= topOutlineCount;
                state.topVirtualRowCount = topVirtualRowCount;
                state.topOutlineCount = topOutlineCount;
                state.rowCount = rowCount;
                state.bottomOutlineCount = bottomOutlineCount;
                state.bottomVirtualRowCount = bottomVirtualRowCount;
                this._updateStateCore();
                return true
            }
            return false
        }
    }, {
        key: "_calcTopRowsInfo",
        value: function(scrollOffset) {
            var top = scrollOffset.top;
            var rowHeight = this.getRowHeight();
            var topVirtualRowCount = Math.floor(top / rowHeight);
            var topOutlineCount = Math.min(topVirtualRowCount, this._getOutlineCount());
            topVirtualRowCount -= topOutlineCount;
            return {
                topVirtualRowCount: topVirtualRowCount,
                topOutlineCount: topOutlineCount
            }
        }
    }, {
        key: "_calcTopRowsDelta",
        value: function(topRowsInfo) {
            var topVirtualRowCount = topRowsInfo.topVirtualRowCount,
                topOutlineCount = topRowsInfo.topOutlineCount;
            var workspace = this.getWorkspace();
            var groupCount = workspace._getGroupCount();
            var totalRowCount = workspace._getTotalRowCount(groupCount);
            return totalRowCount - topVirtualRowCount - topOutlineCount
        }
    }, {
        key: "_calcBottomRowsInfo",
        value: function(topRowsDelta) {
            var workspace = this.getWorkspace();
            var _this$getState = this.getState(),
                pageSize = _this$getState.pageSize;
            var rowCountWithBottom = topRowsDelta >= pageSize ? pageSize : topRowsDelta;
            var bottomVirtualRowCount = topRowsDelta - rowCountWithBottom;
            if (workspace.isGroupedAllDayPanel()) {
                bottomVirtualRowCount -= 1
            }
            var bottomOutlineCount = bottomVirtualRowCount > 0 ? Math.min(bottomVirtualRowCount, this._getOutlineCount()) : 0;
            if (bottomVirtualRowCount > 0) {
                bottomVirtualRowCount -= bottomOutlineCount
            }
            return {
                bottomVirtualRowCount: bottomVirtualRowCount,
                bottomOutlineCount: bottomOutlineCount,
                rowCountWithBottom: rowCountWithBottom
            }
        }
    }, {
        key: "_updateStateCore",
        value: function() {
            var state = this.getState();
            var rowHeight = this.getRowHeight();
            var topVirtualRowCount = state.topVirtualRowCount;
            var bottomVirtualRowCount = state.bottomVirtualRowCount;
            var topOutlineCount = state.topOutlineCount;
            var bottomOutlineCount = state.bottomOutlineCount;
            var prevTopVirtualRowHeight = state.topVirtualRowHeight;
            var prevBottomVirtualRowHeight = state.bottomVirtualRowHeight;
            var prevTopOutlineHeight = state.topOutlineHeight;
            var prevBottomOutlineHeight = state.bottomOutlineHeight;
            var topVirtualRowHeight = rowHeight * topVirtualRowCount;
            var bottomVirtualRowHeight = rowHeight * bottomVirtualRowCount;
            var topOutlineHeight = rowHeight * topOutlineCount;
            var bottomOutlineHeight = rowHeight * bottomOutlineCount;
            var prevTopVirtualHeight = prevTopVirtualRowHeight + prevTopOutlineHeight;
            var topVirtualHeight = topVirtualRowHeight + topOutlineHeight;
            var prevBottomVirtualHeight = prevBottomVirtualRowHeight + prevBottomOutlineHeight;
            var bottomVirtualHeight = bottomVirtualRowHeight + bottomOutlineHeight;
            var isAppend = prevTopVirtualHeight < topVirtualHeight;
            var isPrepend = prevBottomVirtualHeight < bottomVirtualHeight;
            var needAddRows = isAppend || isPrepend;
            if (needAddRows) {
                state.topVirtualRowHeight = topVirtualRowHeight;
                state.bottomVirtualRowHeight = bottomVirtualRowHeight
            }
        }
    }, {
        key: "dispose",
        value: function() {}
    }]);
    return VirtualScrolling
}();
exports.default = VirtualScrolling;
module.exports = exports.default;
