/**
 * DevExtreme (ui/scheduler/workspaces/view_data_generator.js)
 * Version: 20.2.1 (build 20239-0312)
 * Build date: Wed Aug 26 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.default = void 0;

function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread()
}

function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")
}

function _unsupportedIterableToArray(o, minLen) {
    if (!o) {
        return
    }
    if ("string" === typeof o) {
        return _arrayLikeToArray(o, minLen)
    }
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if ("Object" === n && o.constructor) {
        n = o.constructor.name
    }
    if ("Map" === n || "Set" === n) {
        return Array.from(o)
    }
    if ("Arguments" === n || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) {
        return _arrayLikeToArray(o, minLen)
    }
}

function _iterableToArray(iter) {
    if ("undefined" !== typeof Symbol && Symbol.iterator in Object(iter)) {
        return Array.from(iter)
    }
}

function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) {
        return _arrayLikeToArray(arr)
    }
}

function _arrayLikeToArray(arr, len) {
    if (null == len || len > arr.length) {
        len = arr.length
    }
    for (var i = 0, arr2 = new Array(len); i < len; i++) {
        arr2[i] = arr[i]
    }
    return arr2
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}
var ViewDataGenerator = function() {
    function ViewDataGenerator(workspace) {
        _classCallCheck(this, ViewDataGenerator);
        this.workspace = workspace
    }
    _createClass(ViewDataGenerator, [{
        key: "generate",
        value: function() {
            var result;
            if (this.workspace.isVirtualScrolling()) {
                result = this._generateVirtualView()
            } else {
                result = this._generateView()
            }
            return result
        }
    }, {
        key: "_generateVirtualView",
        value: function() {
            var workspace = this._workspace;
            var options = workspace.generateRenderOptions();
            var cellCount = options.cellCount,
                topVirtualRowHeight = options.topVirtualRowHeight,
                bottomVirtualRowHeight = options.bottomVirtualRowHeight,
                startRowIndex = options.startRowIndex,
                groupCount = options.groupCount,
                rowCount = options.rowCount,
                rowCountInGroup = options.rowCountInGroup;
            var groupedData = [];
            var firstGroupIndex = Math.floor(startRowIndex / rowCountInGroup);
            var lastGroupIndex = Math.floor((startRowIndex + rowCount) / rowCountInGroup);
            var isGroupedAllDayPanel = workspace.isGroupedAllDayPanel();
            for (var groupIndex = 0; groupIndex < groupCount; ++groupIndex) {
                var allDayPanelData = [];
                var groupOffset = 0;
                var renderRowCount = rowCount;
                var viewCellsData = [];
                if (groupIndex >= firstGroupIndex && groupIndex <= lastGroupIndex) {
                    var groupRowCount = (groupIndex + 1) * rowCountInGroup;
                    if (startRowIndex + renderRowCount > groupRowCount) {
                        renderRowCount = groupRowCount - startRowIndex
                    } else {
                        var startGroupRowIndex = groupIndex * rowCountInGroup;
                        if (startRowIndex < startGroupRowIndex && startRowIndex + renderRowCount > startGroupRowIndex) {
                            groupOffset = startGroupRowIndex - startRowIndex;
                            renderRowCount -= groupOffset
                        }
                    }
                    viewCellsData = this._generateViewCellsData(options, renderRowCount, startRowIndex, groupOffset);
                    var needRenderAllDayPanel = (startRowIndex + groupOffset) / rowCountInGroup === groupIndex;
                    if (needRenderAllDayPanel) {
                        allDayPanelData = this._generateAllDayPanelData(groupIndex, cellCount)
                    }
                }
                groupedData.push({
                    dateTable: viewCellsData,
                    allDayPanel: allDayPanelData,
                    isGroupedAllDayPanel: isGroupedAllDayPanel
                })
            }
            return {
                viewData: {
                    groupedData: groupedData,
                    isVirtual: true,
                    topVirtualRowHeight: topVirtualRowHeight,
                    bottomVirtualRowHeight: bottomVirtualRowHeight
                },
                viewDataMap: this._getViewDataMap(groupedData)
            }
        }
    }, {
        key: "_generateView",
        value: function() {
            var workspace = this._workspace;
            var options = workspace.generateRenderOptions();
            var isGroupedAllDayPanel = workspace.isGroupedAllDayPanel();
            var rowCount = options.rowCount,
                cellCount = options.cellCount,
                groupCount = options.groupCount;
            var groupedData = [];
            for (var groupIndex = 0; groupIndex < groupCount; ++groupIndex) {
                var viewCellsData = this._generateViewCellsData(options, rowCount, 0, rowCount * groupIndex);
                var allDayPanelData = this._generateAllDayPanelData(groupIndex, cellCount);
                groupedData.push({
                    dateTable: viewCellsData,
                    allDayPanel: allDayPanelData,
                    isGroupedAllDayPanel: isGroupedAllDayPanel
                })
            }
            return {
                viewData: {
                    groupedData: groupedData
                },
                viewDataMap: this._getViewDataMap(groupedData)
            }
        }
    }, {
        key: "_generateViewCellsData",
        value: function(options, renderRowCount) {
            var startRowIndex = arguments.length > 2 && void 0 !== arguments[2] ? arguments[2] : 0;
            var rowOffset = arguments.length > 3 && void 0 !== arguments[3] ? arguments[3] : 0;
            var cellCount = options.cellCount,
                cellDataGetters = options.cellDataGetters;
            var viewCellsData = [];
            var _loop = function(i) {
                var rowIndex = startRowIndex + rowOffset + i;
                viewCellsData.push([]);
                var _loop2 = function(j) {
                    var cellDataValue = {};
                    cellDataGetters.forEach(function(getter) {
                        var cellValue = getter(void 0, rowIndex, j).value;
                        _extends(cellDataValue, cellValue)
                    });
                    viewCellsData[i].push(cellDataValue)
                };
                for (var j = 0; j < cellCount; ++j) {
                    _loop2(j)
                }
            };
            for (var i = 0; i < renderRowCount; ++i) {
                _loop(i)
            }
            return viewCellsData
        }
    }, {
        key: "_generateAllDayPanelData",
        value: function(groupIndex, cellCount) {
            if (!this.workspace.option("showAllDayPanel")) {
                return null
            }
            var rowCount = this.workspace._getRowCount();
            var allDayPanel = [];
            for (var i = 0; i < cellCount; ++i) {
                var rowIndex = Math.max(groupIndex * rowCount, 0);
                var cellDataValue = this.workspace._getAllDayCellData(void 0, rowIndex, i).value;
                allDayPanel.push(cellDataValue)
            }
            return allDayPanel
        }
    }, {
        key: "_getViewDataMap",
        value: function(groupedData) {
            var result = [];
            null === groupedData || void 0 === groupedData ? void 0 : groupedData.forEach(function(_ref) {
                var dateTable = _ref.dateTable,
                    allDayPanel = _ref.allDayPanel,
                    isGroupedAllDayPanel = _ref.isGroupedAllDayPanel;
                isGroupedAllDayPanel && (null === allDayPanel || void 0 === allDayPanel ? void 0 : allDayPanel.length) && result.push(allDayPanel);
                result.push.apply(result, _toConsumableArray(dateTable))
            });
            return result
        }
    }, {
        key: "workspace",
        get: function() {
            return this._workspace
        },
        set: function(value) {
            this._workspace = value
        }
    }]);
    return ViewDataGenerator
}();
exports.default = ViewDataGenerator;
module.exports = exports.default;
